@@ -48,9 +48,6 @@ start_apps([App|Rest]) ->
        start_apps(Rest);
     {error, {already_started, App}} ->
        start_apps(Rest);
-    {error, _Reason} when App =:= public_key ->
-       % ignore on R12B5
-       start_apps(Rest);
     {error, _Reason} ->
        {error, {app_would_not_start, App}}
     end.
